/**
 * Define api of apeman-api-captcha
 * @function define
 * @param {boolean} [options.logging=false] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const spec = require('./spec')
const { randomInt } = require('randomval')

let captchaWrongError = () => Object.assign(new Error('Captcha Wrong'), {
  name: 'CaptchaWrongError',
  status: 400
})

/** @lends define */
function define (images, options = {}) {
  let {
    logging = true
  } = options
  let logger = aslogger({ disabled: !logging })

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Issue a new captcha
     * @method issue
     * @returns {Promise}
     */
    issue () {
      const s = this
      let { session } = s
      return co(function * () {
        let { text, src } = images[ randomInt(0, images.length - 1) ]
        session.captcha = {
          text, src,
          conformed: false,
          issueAt: new Date()
        }
        return { src }
      })
    },

    /**
     * Conform Captcha
     * @method conform
     * @param {String} text
     * @returns {Promise}
     */
    conform (text) {
      const s = this
      let { session } = s
      return co(function * () {
        let { captcha = {} } = session
        let ok = text && (text === captcha.text)
        if (!ok) {
          throw captchaWrongError()
        }
        return ok
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { session } = s
      if (!session) {
        throw new Error('Session not found. You need to add apeman-middleware-session to $middlewares.')
      }
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },

    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
